import json
import os
import subprocess
import zipfile

import toml
from pathlib3x import Path
from playwright.async_api import async_playwright
from rich.console import Console

from worker_automate_hub.config.settings import (
    get_package_version,
    load_env_config,
)
from worker_automate_hub.utils.logger import logger

console = Console()


def write_env_config(env_dict: dict, google_credentials: dict):
    try:
        current_dir = Path.cwd()
        assets_path = current_dir / "assets"
        logs_path = current_dir / "logs"
        assets_path.mkdir(exist_ok=True)
        logs_path.mkdir(exist_ok=True)

        config_file_path = current_dir / "settings.toml"
        config_data = {
            "name": "WORKER",
            "params": {
                "api_base_url": env_dict["API_BASE_URL"],
                "api_auth": env_dict["API_AUTHORIZATION"],
                "notify_alive_interval": env_dict["NOTIFY_ALIVE_INTERVAL"],
                "version": get_package_version("worker-automate-hub"),
                "log_level": env_dict["LOG_LEVEL"],
                "drive_url": env_dict["DRIVE_URL"],
            },
            "google_credentials": google_credentials["content"],
        }

        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)

        log_msg = f"Arquivo de configuração do ambiente criado em {config_file_path}"
        logger.info(log_msg)
        console.print(f"\n{log_msg}\n", style="green")

        return {
            "Message": log_msg,
            "Status": True,
        }
    except Exception as e:
        err_msg = f"Erro ao criar o arquivo de configuração do ambiente. Comando retornou: {e}"
        logger.error(err_msg)
        return {
            "Message": err_msg,
            "Status": False,
        }


def add_worker_config(worker):
    try:
        current_dir = Path.cwd()
        config_file_path = current_dir / "settings.toml"

        if not config_file_path.exists():
            raise FileNotFoundError(
                f"O arquivo de configuração não foi encontrado em: {config_file_path}"
            )

        with open(config_file_path, "r") as config_file:
            config_data = toml.load(config_file)

        config_data["id"] = {
            "worker_uuid": worker["uuidRobo"],
            "worker_name": worker["nomRobo"],
        }

        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)

        log_msg = f"Informações do worker adicionadas ao arquivo de configuração em {config_file_path}"
        console.print(f"\n{log_msg}\n", style="green")
        return {
            "Message": log_msg,
            "Status": True,
        }
    except Exception as e:
        err_msg = f"Erro ao adicionar informações do worker ao arquivo de configuração.\n Comando retornou: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        return {
            "Message": err_msg,
            "Status": False,
        }

async def install_playwright():
    try:
        result1 = subprocess.run(
            ["pipx", "install", "playwright"], check=True, capture_output=True, text=True
        )
        logger.info(result1.stdout)
        result2 = subprocess.run(
            ["playwright", "install"], check=True, capture_output=True, text=True
        )
        logger.info("Playwright instalado com sucesso!")
        logger.info(result2.stdout)
    except subprocess.CalledProcessError as e:
        logger.error(f"Erro ao instalar Playwright: {e}")

def extract_zip(zip_path, extract_to):
    """
    Extrai um arquivo ZIP para um diretório especificado.

    :param zip_path: Caminho para o arquivo ZIP.
    :param extract_to: Diretório onde os arquivos serão extraídos.
    """
    if not os.path.exists(extract_to):
        os.makedirs(extract_to)

    with zipfile.ZipFile(zip_path, "r") as zip_ref:
        zip_ref.extractall(extract_to)
        console.print(f"\nArquivos extraídos para {extract_to}\n", style="green")


async def download_assets_and_extract_from_drive():
    try:
        console.print("\nIniciando download dos assets...\n", style="bold green")
        env_config, creds_loaded = load_env_config()
        creds_loaded = json.loads(creds_loaded)["web"]
        folder_url = env_config["DRIVE_URL"]

        # Diretório de execução atual
        current_dir = Path.cwd()
        output_folder = current_dir / "assets"
        destination_path = current_dir / "temp.zip"

        if not output_folder.exists():
            output_folder.mkdir(parents=True, exist_ok=True)

        await install_playwright()

        async with async_playwright() as p:
            browser = await p.chromium.launch(
                headless=True
            )  # Defina headless=True se não quiser ver o navegador
            context = await browser.new_context()

            # Abrir uma nova página
            page = await context.new_page()

            # Navegar para a URL do arquivo no Google Drive
            await page.goto(folder_url)

            # Aguarde o carregamento da página e clique no botão de download
            await page.wait_for_selector(
                "#drive_main_page > div > div.g3Fmkb > div.S630me > div > div > div > div:nth-child(2) > div > div:nth-child(1) > div > div > div.a-s-tb-sc-Ja-Q.a-s-tb-sc-Ja-Q-Nm.a-Ba-Ed.a-s-Ba-dj > div > div.h-sb-Ic.h-R-d.a-c-d.a-r-d.a-R-d.a-s-Ba-d-Mr-Be-nAm6yf",
                timeout=60000,
            )
            await page.click(
                "#drive_main_page > div > div.g3Fmkb > div.S630me > div > div > div > div:nth-child(2) > div > div:nth-child(1) > div > div > div.a-s-tb-sc-Ja-Q.a-s-tb-sc-Ja-Q-Nm.a-Ba-Ed.a-s-Ba-dj > div > div.h-sb-Ic.h-R-d.a-c-d.a-r-d.a-R-d.a-s-Ba-d-Mr-Be-nAm6yf"
            )

            # Aguarde o download começar
            download = await page.wait_for_event("download")

            # Salve o arquivo no destino especificado
            await download.save_as(destination_path)

            await browser.close()

        extract_zip(destination_path, current_dir)
        os.remove(destination_path)

        console.print("\nAssets baixados com sucesso!\n", style="bold green")
    except Exception as e:
        err_msg = f"Erro ao baixar os assets: {e}"
        logger.error(err_msg)
        console.print(f"\n{err_msg}\n", style="bold red")

async def install_tesseract(setup_path: Path):
    try:
        # Comando para executar com elevação
        command = f'start-process "{setup_path}" -Verb runAs'

        # Executar o comando usando PowerShell
        result = subprocess.run(["powershell", "-Command", command], capture_output=True)
        logger.info(result.stdout)        
        logger.info("Tesseract instalado com sucesso!")
    except subprocess.CalledProcessError as e:
        logger.error(f"Erro ao instalar Tesseract: {e}")

async def download_tesseract():
    # Diretório de execução atual
    current_dir = Path.cwd()
    output_folder = current_dir / "temp"
    destination_path = output_folder / "tesseract.exe"

    folder_url = 'https://github.com/UB-Mannheim/tesseract/wiki'
    async with async_playwright() as p:
            browser = await p.chromium.launch(
                headless=True
            )  # Defina headless=True se não quiser ver o navegador
            context = await browser.new_context()

            # Abrir uma nova página
            page = await context.new_page()

            # Navegar para a URL do arquivo no Google Drive
            await page.goto(folder_url)

            # Aguarde o carregamento da página e clique no botão de download
            await page.wait_for_selector(
                "#wiki-body > div > ul:nth-child(7) > li > a",
                timeout=60000,
            )
            await page.click(
                "#wiki-body > div > ul:nth-child(7) > li > a"
            )

            # Aguarde o download começar
            download = await page.wait_for_event("download")

            # Salve o arquivo no destino especificado
            await download.save_as(destination_path)

            await browser.close()
        
    await install_tesseract(destination_path)
    





import aiohttp


from worker_automate_hub.models.dto.rpa_hitorico_dto import RpaHistoricoDTO
from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.utils.logger import logger


async def store(data: RpaHistoricoDTO) -> dict:    
    env_config, _ = load_env_config()
    
    headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}", "Content-Type": "application/json"}  
    try:
        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            payload = data.model_dump_json()      

            async with session.post(
                f"{env_config["API_BASE_URL"]}/historico",
                data=payload,
                headers=headers_basic,
            ) as response:
                response_text = await response.text()
                logger.info(f"Resposta store: {response_text}")

                if response.status == 200:
                    try:
                        response_data = await response.json()
                        return {
                            "success": response_data,
                            "status_code": response.status,
                        }
                    except aiohttp.ContentTypeError:
                        return {
                            "error": "Resposta não é JSON",
                            "status_code": response.status,
                        }
                else:
                    return {"error": response_text, "status_code": response.status}
    except aiohttp.ClientError as e:
        logger.error(f"Erro de cliente aiohttp: {str(e)}")
        return {"error": str(e), "status_code": 500}
    except Exception as e:
        logger.error(f"Erro inesperado: {str(e)}")
        return {"error": str(e), "status_code": 500}

async def update(data: RpaHistoricoDTO) -> dict:    
    env_config, _ = load_env_config()
    headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}", "Content-Type": "application/json"} 
    try:
        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            payload = data.model_dump_json()

            async with session.put(
                f"{env_config["API_BASE_URL"]}/historico",
                data=payload,
                headers=headers_basic,
            ) as response:
                response_text = await response.text()
                logger.info(f"Resposta update: {response_text}")

                if response.status == 200:
                    try:
                        response_data = await response.json()
                        return {
                            "success": response_data,
                            "status_code": response.status,
                        }
                    except aiohttp.ContentTypeError:
                        return {
                            "error": "Resposta não é JSON",
                            "status_code": response.status,
                        }
                else:
                    return {"error": response_text, "status_code": response.status}
    except aiohttp.ClientError as e:
        logger.error(f"Erro de cliente aiohttp: {str(e)}")
        return {"error": str(e), "status_code": 500}
    except Exception as e:
        logger.error(f"Erro inesperado: {str(e)}")
        return {"error": str(e), "status_code": 500}



from rich.console import Console

from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.updater import check_for_update

console = Console()


async def handle_api_response(response, last_alive=False):

    status = response.status
    match status:
        case 200:
            if last_alive:
                console.print(
                    "\n[Worker last alive] Informado salvo com sucesso.\n",
                    style="bold green",
                )
            else:
                data = await response.json()
                return {"data": data, "update": False}
        case 204:
            console.print("204 - Nenhum processo encontrado", style="yellow")
            logger.info("204 - Nenhum processo encontrado")
        case 300:
            console.print("300 - Necessário atualização!", style="blue")
            logger.info("300 - Necessário atualização!")
            check_for_update()
        case 401:
            console.print("401 - Acesso não autorizado!", style="red")
            logger.error("401 - Acesso não autorizado!")
        case 404:
            console.print("404 - Nenhum processo disponível!", style="yellow")
            logger.error("404 - Nenhum processo disponível!")
        case 500:
            console.print("500 - Erro interno da API!", style="red")
            logger.error("500 - Erro interno da API!")
        case _:
            logger.error(f"Status não tratado: {status}")
    return None



import asyncio
import re
import time
import warnings
from datetime import datetime

import pyautogui
import pyperclip
from pywinauto.application import Application
from rich.console import Console

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    api_simplifica,
    find_element_center,
    find_target_position,
    kill_process,
    login_emsys,
    take_screenshot,
    take_target_position,
    type_text_into_field,
)

console = Console()

ASSETS_BASE_PATH = 'assets/descartes_transferencias_images/'
ALMOXARIFADO_DEFAULT = "50"

async def descartes(task):
    try:
        #Inicializa variaveis
        pre_venda_message = None
        nota_fiscal = None
        log_msg = None
        valor_nota = None
        #Get config from BOF
        console.print("Obtendo configuração...\n")
        config = await get_config_by_name("Descartes_Emsys")
        itens = task['configEntrada']['itens']

        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()

        # Print da resolução
        console.print(f"Largura: {screen_width}, Altura: {screen_height}")

        # Abre um novo emsys        
        await kill_process("EMSys")
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config['conConfiguracao'], app, task)
        
        if return_login['sucesso'] == True:
            console.print("Pesquisando por: Cadastro Pré Venda")
            type_text_into_field('Cadastro Pré Venda', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            time.sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Cadastro Pre Venda' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login["retorno"]}")
            console.print(f"\nError Message: {return_login["retorno"]}", style="bold red")
            return return_login

        time.sleep(7)

        #Preenche data de validade
        console.print("Preenchendo a data de validade...\n")
        screenshot_path = take_screenshot()
        target_pos = (961, 331) #find_target_position(screenshot_path, "Validade", 10, 0, 15) 
        if target_pos == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo de validade"}
        
        pyautogui.click(target_pos)
        pyautogui.write(f'{datetime.now().strftime("%d/%m/%Y")}', interval=0.1)
        pyautogui.press('tab')
        console.print(f"\nValidade Digitada: '{datetime.now().strftime("%d/%m/%Y")}'\n", style="bold green")
        time.sleep(1)
        
        #Condição da Pré-Venda 
        console.print("Selecionando a Condição da Pré-Venda\n")       
        condicao_field = find_target_position(screenshot_path, "Condição", 10, 0, 15) 
        if condicao_field == None:
            condicao_field = (1054, 330)
            
        pyautogui.click(condicao_field)
        time.sleep(1)
        pyautogui.write("A")
        time.sleep(1)
        pyautogui.press("down")
        pyautogui.press("enter")
        time.sleep(1)
        
        #Preenche o campo do cliente com o número da filial
        console.print("Preenchendo o campo do cliente com o número da filial...\n")
        cliente_field_position = await find_element_center(ASSETS_BASE_PATH + "field_cliente.png", (795, 354, 128, 50), 10)
        if cliente_field_position == None:
            cliente_field_position = (884, 384)  

        pyautogui.click(cliente_field_position)
        pyautogui.hotkey("ctrl", "a")
        pyautogui.hotkey("del")
        pyautogui.write(task['configEntrada']['filialEmpresaOrigem'])
        pyautogui.hotkey("tab")
        time.sleep(6)

        #Verifica se precisa selecionar endereço
        console.print("Verificando se precisa selecionar endereço...\n")
        screenshot_path = take_screenshot()
        window_seleciona_endereco_position = take_target_position(screenshot_path, "Endereço")
        if window_seleciona_endereco_position is not None:
            log_msg = f'Aviso para selecionar Endereço'
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
            return {"sucesso": False, "retorno": log_msg}
        else:
            log_msg = "Sem Aviso de Seleção de Endereço"
            console.print(log_msg, style='bold green')
            logger.info(log_msg)
        
        # Clica em cancelar na Janela "Busca Representante"
        console.print("Cancelando a Busca Representante\n")
        screenshot_path = take_screenshot()
        window_busca_representante_position = take_target_position(screenshot_path, "Representante")
        if window_busca_representante_position is not None:
            button_cancelar_position = find_target_position(screenshot_path, "Cancelar", attempts=15)            
            pyautogui.click(button_cancelar_position)
        
        time.sleep(2)

        # Aviso "Deseja alterar a condição de pagamento informada no cadastro do cliente?"
        console.print("Verificando alerta de alteração de pagamento informada no cadastro do cliente...\n")
        screenshot_path = take_screenshot()
        payment_condition_warning_position = take_target_position(screenshot_path, "pagamento")
        if payment_condition_warning_position is not None:
            button_no_position = (999, 568) #find_target_position(screenshot_path, "No", attempts=15)            
            pyautogui.click(button_no_position)
            console.print(f"\nClicou 'No' Mensagem 'Deseja alterar a condição de pagamento informada no cadastro do cliente?'", style="bold green")
            time.sleep(6)
        else:
            log_msg = f"\nError Message: Aviso de condição de pagamento não encontrado"
            logger.info(log_msg)
            console.print(log_msg, style="bold red")
        time.sleep(3)

        #Seleciona 'Custo Médio' (Seleção do tipo de preço)
        console.print("Seleciona 'Custo Médio' (Seleção do tipo de preço)...\n")
        screenshot_path = take_screenshot()
        custo_medio_select_position = find_target_position(screenshot_path, "Médio", attempts=15)

        if custo_medio_select_position == None:
            custo_medio_select_position = (851, 523)

        if custo_medio_select_position is not None:
            pyautogui.click(custo_medio_select_position)
            button_ok_position = (1042, 583) #find_target_position(screenshot_path, "OK", attempts=15)            
            pyautogui.click(button_ok_position)
            time.sleep(5)
            console.print(f"\nClicou OK 'Custo médio'", style="bold green")
        else:
            log_msg = f"\nError Message: Campo 'Custo Médio' não encontrado"
            logger.info(log_msg)
            console.print(log_msg, style="bold yellow")

        time.sleep(8)        

        #Clica em ok na mensagem "Existem Pré-Vendas em aberto para este cliente."
        console.print("Clica em ok na mensagem 'Existem Pré-Vendas em aberto para este cliente.'\n")
        screenshot_path = take_screenshot()
        existing_pre_venda_position = find_target_position(screenshot_path, "Existem", attempts=15)

        if existing_pre_venda_position == None:
            existing_pre_venda_position = await find_element_center(ASSETS_BASE_PATH + "existing_pre_venda.png", (831, 437, 247, 156), 15)

        if existing_pre_venda_position is not None:
            button_ok_position = (962, 562)
            pyautogui.click(button_ok_position)
            console.print(f"\nClicou OK 'Pre Venda Existente'", style="bold green")
            time.sleep(5)
        else:
            log_msg = f"\nError Message: Menssagem de prevenda existente não encontrada"
            logger.info(log_msg)
            console.print(log_msg, style="bold yellow")        

        #Define representante para "1"
        console.print("Definindo representante para '1'\n")
        screenshot_path = take_screenshot()
        field_representante_position = find_target_position(screenshot_path, "Representante", 0, 50, attempts=15)

        if field_representante_position == None:
            field_representante_position = await find_element_center(ASSETS_BASE_PATH + "field_representante.png", (679, 416, 214, 72), 15)
            if field_representante_position is not None:
                lista = list(field_representante_position)
                lista[0] += 50
                lista[1] += 1
                field_representante_position = tuple(lista)

        if field_representante_position is not None:
            pyautogui.doubleClick(field_representante_position)
            pyautogui.hotkey("ctrl", "a")
            pyautogui.hotkey("del")
            pyautogui.write('1')
            pyautogui.hotkey("tab")
        
        time.sleep(3)

        #Seleciona modelo de capa
        console.print("Selecionando o modelo de capa...")
        screenshot_path = take_screenshot()
        model_descarte_position = find_target_position(screenshot_path, "Modelo", 0, 100, attempts=15)

        if model_descarte_position == None:
            model_descarte_position = await find_element_center(ASSETS_BASE_PATH + "field_modelo_faturamento.png", (681, 489, 546, 96), 15)
            if model_descarte_position is not None:
                lista = list(model_descarte_position)
                lista[0] += 100
                model_descarte_position = tuple(lista)

        if model_descarte_position == None:
            model_descarte_position = (848, 527)

        if model_descarte_position is not None:
            pyautogui.click(model_descarte_position)
            pyautogui.click(1500, 800)
            pyautogui.write("B")
            pyautogui.hotkey("tab")
        else:
            log_msg = f'Campo Modelo na capa da nota não encontrado'
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
            return {"sucesso": False, "retorno": log_msg}

        #Abre Menu itens
        console.print("Abrindo Menu Itens...\n")
        menu_itens = await find_element_center(ASSETS_BASE_PATH + "menu_itens.png", (526, 286, 152, 45), 10)

        if menu_itens == None:
            menu_itens = (570, 317)

        if menu_itens is not None:
            pyautogui.click(menu_itens)
        else:
            log_msg = f'Campo "Itens" no menu da pré-venda não encontrado'
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
            return {"sucesso": False, "retorno": log_msg}

        time.sleep(2)

        #Loop de itens
        console.print("Inicio do loop de itens\n")
        for item in itens:
            screenshot_path = take_screenshot()
            #Clica no botão inclui para abrir a tela de item
            console.print("Clicando em Incluir...\n")
            button_incluir = (905, 573) #find_target_position(screenshot_path, "Incluir", 0, 0, attempts=15)
            if button_incluir is not None:
                pyautogui.click(button_incluir)
                console.print("\nClicou em 'Incluir'", style='bold green')            
            else:
                log_msg = f'Botão "Incluir" não encontrado'
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                return {"sucesso": False, "retorno": log_msg}
            time.sleep(3)
            
            # Digita Almoxarifado
            console.print("Preenchendo o campo de almoxarifado...\n")
            screenshot_path = take_screenshot()
            field_almoxarifado = (839, 313) #find_target_position(screenshot_path, "Almoxarifado",0, 129, 15)
            if field_almoxarifado is not None:
                pyautogui.doubleClick(field_almoxarifado)
                pyautogui.hotkey('del')
                pyautogui.write(task['configEntrada']['filialEmpresaOrigem'] + ALMOXARIFADO_DEFAULT)
                pyautogui.hotkey('tab')
                time.sleep(2)
                console.print(f"\nDigitou almoxarifado {task['configEntrada']['filialEmpresaOrigem'] + ALMOXARIFADO_DEFAULT}", style='bold green')            
            else:
                log_msg = f'Campo Almoxarifado não encontrado'
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                return {"sucesso": False, "retorno": log_msg}

            #Segue para o campo do item
            console.print("Preenchendo o campo do item...\n")
            field_item = (841, 339) #find_target_position(screenshot_path, "Item", 0, 130, 15)
            if field_item is not None:
                pyautogui.doubleClick(field_item)
                pyautogui.hotkey('del')
                pyautogui.write(item['codigoProduto'])
                pyautogui.hotkey('tab')
                time.sleep(2)
                console.print(f"\nDigitou item {item['codigoProduto']}", style='bold green')
            else:
                log_msg = f'Campo Item não encontrado.'
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                return {"sucesso": False, "retorno": log_msg}


            #Checa tela de pesquisa de item
            console.print("Verificando a existencia da tela de pesquisa de item...\n")
            screenshot_path = take_screenshot()
            window_pesquisa_item = await find_element_center(ASSETS_BASE_PATH + "window_pesquisa_item.png", (488, 226, 352, 175), 10)
            console.print(f"Produto {item['codigoProduto']} encontrado", style="bold green")
            logger.info(f"Produto {item['codigoProduto']} encontrado")

            if window_pesquisa_item is not None:
                observacao = f"Item {item['codigoProduto']} não encontrado, verificar cadastro"
                console.print(f"{observacao}", style="bold green")
                logger.info(f"{observacao}")
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                return {"sucesso": False, "retorno": observacao}
                
            #Checa se existe alerta de item sem preço, se existir retorna erro(simplifica e bof)
            console.print("Verificando se existe alerta de item sem preço, se existir retorna erro(simplifica e bof)...\n")
            warning_price = await find_element_center(ASSETS_BASE_PATH + "warning_item_price.png",  (824, 426, 255, 191), 10)
            if warning_price is not None:
                observacao = f"Item {item['codigoProduto']} não possui preço, verificar erro de estoque ou de bloqueio."
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", observacao, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                return {"sucesso": False, "retorno": observacao}
            
            screenshot_path = take_screenshot()
            
            time.sleep(2)

            #Seleciona o Saldo Disponivel e verifica se ah possibilidade do descarte
            console.print("Selecionando o Saldo Disponivel e verificando se há possibilidade do descarte...\n")
            screenshot_path = take_screenshot()
            field_saldo_disponivel = (916, 606) #find_target_position(screenshot_path + "Saldo", 20, 0, 10)
            if field_saldo_disponivel is not None:
                pyautogui.doubleClick(field_saldo_disponivel)
                time.sleep(1)
                pyautogui.doubleClick(field_saldo_disponivel)
                time.sleep(1)
                pyautogui.hotkey('ctrl', 'c')
                amount_avaliable= ''
                amount_avaliable = pyperclip.paste()
                console.print(f"Saldo Disponivel: '{amount_avaliable}'", style="bold green")

                #Verifica se o saldo disponivel é valido para descartar
                if int(amount_avaliable) > 0 and int(amount_avaliable) >= int(item['qtd']): 
                    field_quantidade = (1047, 606) #find_target_position(screenshot_path, "Quantidade", 20, 0, 15)
                    pyautogui.doubleClick(field_quantidade)
                    pyautogui.hotkey('del')
                    pyautogui.write(str(item['qtd']))
                    pyautogui.hotkey('tab')
                    time.sleep(2)
                else:
                    log_msg = f"Saldo disponivel: '{amount_avaliable}' é menor que '{item['qtd']}' o valor que deveria ser descartado. Item: '{item['codigoProduto']}'"
                    await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                    console.print(log_msg, style="bold red")
                    return {"sucesso": False, "retorno": log_msg}

            #Clica em incluir para adicionar o item na nota
            console.print("Clicando em incluir para adicionar o item na nota...\n")
            button_incluir_item = (1007, 745) #find_target_position(screenshot_path, "Inlcuir", 0, 0, 15)
            if button_incluir_item is not None:
                pyautogui.click(button_incluir_item)
                time.sleep(2)
            else:
                log_msg = f"Botao 'Incluir' item não encontrado"
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                console.print(log_msg, style="bold red")
                return {"sucesso": False, "retorno": log_msg}
            
            #Clica em cancelar para fechar a tela e abrir novamente caso houver mais itens
            console.print("Clicando em cancelar para fechar a tela e abrir novamente caso houver mais itens...\n")
            button_cancela_item = (1194, 745) #find_target_position(screenshot_path, "Cancela", 0, 0, 15)
            if button_cancela_item is not None:
                pyautogui.click(button_cancela_item)
                time.sleep(2)
            else:
                log_msg = f"Botao cancelar para fechar a tela do item nao encontrado"
                await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
                console.print(log_msg, style="bold red")
                return {"sucesso": False, "retorno": log_msg}
            
        time.sleep(2)

        #Clica no botão "+" no canto superior esquerdo para lançar a pre-venda
        console.print("Clica no botão '+' no canto superior esquerdo para lançar a pre-venda")
        #Precisa manter por imagem pois não tem texto
        button_lanca_pre_venda = await find_element_center(ASSETS_BASE_PATH + "button_lanca_prevenda.png", (490, 204, 192, 207), 15)
        if button_lanca_pre_venda is not None:
            pyautogui.click(button_lanca_pre_venda.x, button_lanca_pre_venda.y)
            console.print("\nLançou Pré-Venda", style="bold green")
        else:
            log_msg = f"Botao lança pre-venda nao encontrado"
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
            console.print(log_msg, style="bold red")
            return {"sucesso": False, "retorno": log_msg}
        
        time.sleep(5)

        screenshot_path = take_screenshot()

       #Verifica mensagem de "Pré-Venda incluida com número: xxxxx"
        console.print("Verificando mensagem de 'Pré-Venda incluida com número: xxxxx'...\n")
        included_pre_venda = find_target_position(screenshot_path, "incluída", attempts=15)
        if included_pre_venda is not None:
            #Clica no centro da mensagem e copia o texto para pegar o numero da pre-venda
            pyautogui.click(included_pre_venda)
            pyautogui.hotkey("ctrl", "c")
            pre_venda_message = pyperclip.paste()
            pre_venda_message = re.findall(r'\d+-\d+', pre_venda_message)
            console.print(f"Numero pré-venda: '{pre_venda_message[0]}'",style='bold green')
            #Clica no ok da mensagem
            button_ok = (1064, 604) #find_target_position(screenshot_path, "Ok", 15)
            pyautogui.click(button_ok)
        else:
            log_msg = f"Não achou mensagem Pré-Venda incluida."
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
            console.print(log_msg, style="bold red")
            return {"sucesso": False, "retorno": log_msg}
        
        screenshot_path = take_screenshot()

        #Message 'Deseja pesquisar pré-venda?'
        console.print("Verificando a existencia da mensagem: 'Deseja pesquisar pré-venda?'...\n")
        message_prevenda = take_target_position(screenshot_path, "Deseja")
        if message_prevenda is not None:
            button_yes = find_target_position(screenshot_path, "Yes", attempts=15)
            pyautogui.click(button_yes)
        else:
            log_msg = f"Mensagem 'Deseja pesquisar pré-venda?' não encontrada."
            console.print(log_msg, style="bold yellow")
        
        screenshot_path = take_screenshot()
        #Confirma pré-venda
        #Pode não precisar em descartes, mas em trânsferencias é obrigatório
        console.print("Confirmando a Pre-Venda...\n")
        button_confirma_transferencia = take_target_position(screenshot_path, "confirma")
        if button_confirma_transferencia is not None:
            pyautogui.click(button_confirma_transferencia)
            console.log("Confirmou transferencia", style="bold green")
        else:
            log_msg = f"Botao 'Confirma' não encontrado"
            console.print(log_msg, style="bold yellow")
        
        pyautogui.moveTo(1200, 300)

        console.print("Verificando a mensagem: Confirmar transferencia...\n")
        screenshot_path = take_screenshot()
        message_confirma_transferencia = take_target_position(screenshot_path, "confirmar")
        if message_confirma_transferencia is not None:
            #clica em sim na mensagem
            button_yes= find_target_position(screenshot_path, "Yes", attempts=15)
            pyautogui.click(button_yes)
            console.log("Cliclou em 'Sim' para cofirmar a pré-venda", style='bold green')
            pyautogui.moveTo(1200, 300)
            time.sleep(2)
            screenshot_path = take_screenshot()
            vencimento_message_primeira_parcela = take_target_position(screenshot_path, "vencimento")
            #TODO apareceu em dev apenas pode nao ser necesário em prod mantive por segurança
            #Pode nao aparecer na prod
            if vencimento_message_primeira_parcela is not None:
                button_yes = find_target_position(screenshot_path, "Yes", attempts=15)
                pyautogui.click(button_yes)
            time.sleep(2)
            screenshot_path = take_screenshot()
            #Clica no OK 'Pre-Venda incluida com sucesso'
            button_ok = find_target_position(screenshot_path, "Ok", attempts=15)
            pyautogui.click(button_ok)
            console.log("Cliclou em 'OK' para pré-venda confirmada com sucesso", style='bold green')
        else:
            log_msg = f"Mensagem 'Deseja realmente confirmar esta pré-venda?' não encontrada."
            console.print(log_msg, style="bold yellow")

        pyautogui.moveTo(1000, 500)

        #Clica em Faturar
        console.print("Clicando em Faturar...\n")
        screenshot_path = take_screenshot()
        button_faturar = (1313, 395) #find_target_position(screenshot_path, "Faturar", attempts=15)
        if button_faturar is not None:
            time.sleep(2)
            pyautogui.click(button_faturar)
            console.print(f"Clicou em: 'Faturar'",style='bold green')
        else:
            log_msg = f"Não encontrou botão faturar"
            console.print(log_msg, style="bold yellow")
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
            return {"sucesso": False, "retorno": f'{log_msg}'}
         
        time.sleep(10)


        #Verifica se existe a mensagem de recalcular parcelas
        console.print("Verificando se existe a mensagem de recalcular parcelas...\n")
        screenshot_path = take_screenshot()
        message_recalcular = find_target_position(screenshot_path, "Recalcular", attempts=15)
        #Se existir clica em nao
        if message_recalcular is not None:
            button_no = (999, 560) #find_target_position(screenshot_path, "No", attempts=15)
            pyautogui.click(button_no)
            console.log("Cliclou em 'No' na mensagem de recalcular parcelas", style="bold green") 
        else:
            logger.info(f"Mensagem de para recalcular parcelas da pre-venda nao existe")
            console.print(f"Mensagem de para recalcular parcelas da pre-venda nao existe", style="bold yellow")
        
        time.sleep(8)

        #TODO apareceu em dev apenas pode nao ser necesário em prod mantive por segurança
        # Warning 'Atribuir o valor da Substituição tributária na Primeira Parcela?'
        console.print("Verificando se existe a mensagem: 'Atribuir o valor da Substituição tributária na Primeira Parcela?'...\n")
        screenshot_path = take_screenshot()
        warning_substituicao_trib = take_target_position(screenshot_path, "tributária")
        if warning_substituicao_trib is not None:
            console.log("Vai clicar em não")
            button_no = find_target_position(screenshot_path,"Não", attempts=15)
            pyautogui.click(button_no)
            console.log("Cliclou em 'No' na mensagem de Substiucao tributaria", style="bold green")
        else:
            console.log("Sem aviso apresentado! 'Atribuir o valor da Substituição tributária na Primeira Parcela?'", style="bold green")

        time.sleep(8)
        console.print("Selecionando o modelo...\n")
        modelo_select_position = (874,267)
        pyautogui.click(modelo_select_position)
        pyautogui.write("N")
        time.sleep(1)
        for _ in range(3):
            pyautogui.press('down')
        pyautogui.press('enter')
        
        time.sleep(5)

        #Pega total da Nota
        #Vou manter no antigo ainda para facilitar, pois existe mais de um campo "total" na tela
        console.print("Obtendo o total da Nota...\n")
        field_total_nota = await find_element_center(ASSETS_BASE_PATH + "field_total_nota.png", (1067, 767, 306, 98), 15)
        if field_total_nota is not None:
            pyautogui.click(field_total_nota.x, field_total_nota.y)
            pyautogui.doubleClick(field_total_nota.x + 35, field_total_nota.y)
            pyautogui.hotkey("ctrl", "c")
            valor_nota = pyperclip.paste()
            valor_nota = re.findall(r'\b\d{1,3}(?:\.\d{3})*,\d{2}\b', valor_nota)
            console.print(f"\nValor NF: '{valor_nota[0]}'",style='bold green')
        else:
            console.print(f"\nNão conseguiu extrair o valor da nota",style='bold red')
            logger.info("\nNão conseguiu extrair o valor da nota")

        #Clicar no botao "OK" com um certo verde
        console.print("Clicando no OK...\n")
        screenshot_path = take_screenshot()
        button_verde = (1180, 822) #find_target_position(screenshot_path, "Ok", attempts=15)
        if button_verde is not None:
            pyautogui.click(button_verde)
        else:
            log_msg = f"Não conseguiu achar botão 'OK'"
            console.print(log_msg)
            logger.info(log_msg)
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
            return {"sucesso": False, "retorno": log_msg}
    
        time.sleep(5)

        #Aviso "Deseja faturar pré-venda?"
        console.print("Verificando a mensagem: 'Deseja faturar pré-venda?'...\n")
        screenshot_path = take_screenshot()
        faturar_pre_venda = find_target_position(screenshot_path, "faturar", attempts=15)
        if faturar_pre_venda is not None:
            button_yes = (918, 561) #find_target_position(screenshot_path, "yes", attempts=15)
            pyautogui.click(button_yes)
        else:
            log_msg = {
            "numero_pre_venda": pre_venda_message[0],
            "numero_nota": ''
            }
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal[0], valor_nota[0])
            return {"sucesso": False, "retorno": "Falha ao cliclar em: 'SIM' no aviso: 'Deseja realmente faturar esta Pré-Venda ?'"}


        #Mensagem de nota fiscal gerada com número
        console.print("Verificando a mensagem: 'nota fiscal gerada com número'...\n")
        screenshot_path = take_screenshot()
        nota_fiscal_gerada = find_target_position(screenshot_path, "gerada", attempts=15)
        pyautogui.click(nota_fiscal_gerada)
        pyautogui.hotkey("ctrl", "c")
        nota_fiscal = pyperclip.paste()
        nota_fiscal = re.findall(r'\d+-?\d*', nota_fiscal)
        console.print(f"\nNumero NF: '{nota_fiscal[0]}'",style='bold green')

        time.sleep(7)

        #Transmitir a nota
        console.print("Transmitindo a nota...\n")
        screenshot_path = take_screenshot()
        transmitir = find_target_position(screenshot_path, "transmitir", attempts=15)
        if transmitir is not None:
            pyautogui.click(transmitir)
            logger.info("\nNota Transmitida")
            console.print("\nNota Transmitida", style="bold green")
        else:
            log_msg = f'\nNota não Transmitida'
            logger.info(log_msg)
            console.print(log_msg ,style="bold red")
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal[0], valor_nota[0])
            return {"sucesso": False, "retorno": log_msg}

        time.sleep(7)

        #Fechar transmitir nota
        console.print("Fechando a transmissão da nota...\n")
        screenshot_path = take_screenshot()
        transmitir_fechar = find_target_position(screenshot_path, "fechar", attempts=15)
        if transmitir_fechar is not None:
            pyautogui.click(transmitir_fechar)
            log_msg = f'Nota Transmitida com sucesso'
            logger.info(log_msg)
            console.print(log_msg)
        else:
            log_msg = f'Nota não transmitida'
            logger.info(log_msg)
            console.print(log_msg, style='bold red')
            await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal[0], valor_nota[0])
            return {"sucesso": False, "retorno": log_msg}

        log_msg = {
            "numero_pre_venda": pre_venda_message[0],
            "numero_nota": nota_fiscal[0],
            "valor_nota": valor_nota[0]
        }

        await api_simplifica(task['configEntrada']['urlRetorno'], "SUCESSO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal[0], valor_nota[0])
        return {"sucesso": True, "retorno": log_msg}
    
    except Exception as ex:
        log_msg = f"Erro Processo Descartes: {ex}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal[0], valor_nota[0])
        return {"sucesso": False, "retorno": log_msg}




import asyncio
import math
import os
import subprocess
import time
import warnings
from pathlib import Path

import aiohttp
import cv2
import psutil
import pyautogui
from PIL import Image
from pytesseract import pytesseract
from rich.console import Console

from worker_automate_hub.config.settings import (
    load_env_config,
    load_worker_config,
)
from worker_automate_hub.utils.logger import logger

console = Console()


async def get_system_info():
    worker_config = load_worker_config()
    max_cpu = psutil.cpu_percent(interval=10.0)
    cpu_percent = psutil.cpu_percent(interval=1.0)
    memory_info = psutil.virtual_memory()

    return {
        "uuidRobo": worker_config["UUID_ROBO"],
        "maxCpu": f"{max_cpu}",
        "maxMem": f"{memory_info.total / (1024 ** 3):.2f}",
        "usoCpu": f"{cpu_percent}",
        "usoMem": f"{memory_info.used / (1024 ** 3):.2f}",
        "situacao": "{'status': 'em desenvolvimento'}",
    }


async def get_new_task_info():
    env_config, _ = load_env_config()
    worker_config = load_worker_config()
    return {
        "uuidRobo": worker_config["UUID_ROBO"],
        "versao": env_config["VERSION"],
    }


async def kill_process(process_name: str):
    try:
        # Obtenha o nome do usuário atual
        current_user = os.getlogin()

        # Liste todos os processos do sistema
        result = await asyncio.create_subprocess_shell(
            f'tasklist /FI "USERNAME eq {current_user}" /FO CSV /NH',
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        stdout, stderr = await result.communicate()

        if result.returncode != 0:
            err_msg = f"Erro ao listar processos: {stderr.decode().strip()}"
            logger.error(err_msg)
            console.print(err_msg, style="bold red")
            return

        if stdout:
            lines = stdout.decode().strip().split("\n")
            for line in lines:
                # Verifique se o processo atual corresponde ao nome do processo
                if process_name in line:
                    try:
                        # O PID(Process ID) é a segunda coluna na saída do tasklist
                        pid = int(line.split(",")[1].strip('"'))
                        await asyncio.create_subprocess_exec(
                            "taskkill", "/PID", str(pid), "/F"
                        )
                        log_msg = f"Processo {process_name} (PID {pid}) finalizado."
                        logger.info(log_msg)
                        console.print(
                            f"\n{log_msg}\n",
                            style="bold green",
                        )
                    except Exception as ex:
                        err_msg = f"Erro ao tentar finalizar o processo {process_name} (PID {pid}): {ex}"
                        logger.error(err_msg)
                        console.print(
                            err_msg,
                            style="bold red",
                        )
        else:
            log_msg = f"Nenhum processo chamado {process_name} encontrado para o usuário {current_user}."
            logger.info(
                log_msg,
                None,
            )
            console.print(
                log_msg,
                style="bold yellow",
            )

    except Exception as e:
        err_msg = f"Erro ao tentar matar o processo: {e}"
        logger.error(err_msg)
        console.print(err_msg, style="bold red")


async def find_element_center(image_path, region_to_look, timeout):
    try:
        counter = 0
        confidence_value = 1.00
        grayscale_flag = False

        while counter <= timeout:
            try:
                element_center = pyautogui.locateCenterOnScreen(
                    image_path,
                    region=region_to_look,
                    confidence=confidence_value,
                    grayscale=grayscale_flag,
                )
            except Exception as ex:
                element_center = None
                console.print(
                    f"[{counter+1}] - Elemento não encontrado na posição: {region_to_look}"
                )

            if element_center:
                console.print(
                    f"[{counter+1}] - Elemento encontrado na posição: {region_to_look}\n",
                    style="green",
                )
                return element_center
            else:
                counter += 1

                if confidence_value > 0.81:
                    confidence_value -= 0.01

                if counter >= math.ceil(timeout / 2):
                    grayscale_flag = True

                await asyncio.sleep(1)

        return None
    except Exception as ex:
        console.print(
            f"{counter} - Buscando elemento na tela: {region_to_look}",
            style="bold yellow",
        )
        return None


def type_text_into_field(text, field, empty_before, chars_to_empty):
    try:
        if empty_before:
            field.type_keys("{BACKSPACE " + chars_to_empty + "}", with_spaces=True)

        field.type_keys(text, with_spaces=True)

        if str(field.texts()[0]) == text:
            return
        else:
            field.type_keys("{BACKSPACE " + chars_to_empty + "}", with_spaces=True)
            field.type_keys(text, with_spaces=True)

    except Exception as ex:
        logger.error("Erro em type_text_into_field: " + str(ex), None)
        console.print(f"Erro em type_text_into_field: {str(ex)}", style="bold red")


async def wait_element_ready_win(element, trys):
    max_trys = 0

    while max_trys < trys:
        try:
            if element.wait("exists", timeout=2):
                await asyncio.sleep(1)
                if element.wait("exists", timeout=2):
                    await asyncio.sleep(1)
                    if element.wait("enabled", timeout=2):
                        element.set_focus()
                        await asyncio.sleep(1)
                        if element.wait("enabled", timeout=1):
                            return True

        except Exception as ex:
            logger.error("wait_element_ready_win -> " + str(ex), None)
            console.print(
                f"Erro em wait_element_ready_win: {str(ex)}", style="bold red"
            )

        max_trys = max_trys + 1

    return False


async def login_emsys(config, app, task):

    from pywinauto.application import Application

    warnings.filterwarnings(
        "ignore",
        category=UserWarning,
        message="32-bit application should be automated using 32-bit Python",
    )

    await asyncio.sleep(10)
    # Testa se existe alguma mensagem no Emsys
    console.print("Testando se existe alguma mensagem no Emsys...")
    window_message_login_emsys = await find_element_center(
        "assets/emsys/window_message_login_emsys.png", (560, 487, 1121, 746), 15
    )

    # Clica no "Não mostrar novamente" se existir
    console.print("Clicando no 'Não mostrar novamente' se existir...")
    if window_message_login_emsys:
        pyautogui.click(window_message_login_emsys.x, window_message_login_emsys.y)
        pyautogui.click(
            window_message_login_emsys.x + 383, window_message_login_emsys.y + 29
        )
        console.print("Mensagem de login encontrada e fechada.", style="bold green")

    # Ve se o Emsys esta aberto no login
    console.print("Verificando se o Emsys esta aberto no login...")
    image_emsys_login = await find_element_center(
        "assets/emsys/logo_emsys_login.png", (800, 200, 1400, 700), 60
    )

    if image_emsys_login:
        console.print("Aguardando a janela de login ficar pronta...")
        if await wait_element_ready_win(app["Login"]["Edit2"], 80):
            console.print("Procurando o icone disconect_database...")
            disconect_database = await find_element_center(
                "assets/emsys/disconect_database.png", (1123, 452, 1400, 578), 60
            )

            if disconect_database:
                # Realiza login no Emsys
                console.print("Realizando login no Emsys...")
                type_text_into_field(config["user"], app["Login"]["Edit2"], True, "50")
                pyautogui.press("tab")
                type_text_into_field(
                    config["pass"],
                    app["Login"]["Edit1"],
                    True,
                    "50",
                )
                pyautogui.press("enter")

                # Seleciona a filial do emsys
                console.print("Seleciona a filial do emsys...")
                selecao_filial = await find_element_center(
                    "assets/emsys/selecao_filial.png", (480, 590, 820, 740), 15
                )

                console.print(f"Selecao filial via imagem: {selecao_filial}")
                if selecao_filial == None:
                    screenshot_path = take_screenshot()
                    selecao_filial = find_target_position(
                        screenshot_path, "Grupo", 0, -50, attempts=15
                    )
                    console.print(f"Selecao filial localização de texto: {selecao_filial}")
                    if selecao_filial == None:
                        selecao_filial = (700, 639)
                        console.print(f"Selecao filial posição fixa: {selecao_filial}")

                    pyautogui.click(selecao_filial)
                    console.print(f"Escrevendo [{task["configEntrada"]["filialEmpresaOrigem"]}] no campo filial...")
                    pyautogui.write(task["configEntrada"]["filialEmpresaOrigem"])
                    

                else:                    
                    console.print(f"Escrevendo [{task["configEntrada"]["filialEmpresaOrigem"]}] no campo filial...")
                    type_text_into_field(
                        task["configEntrada"]["filialEmpresaOrigem"],
                        app["Seleção de Empresas"]["Edit"],
                        True,
                        "50",
                    )
                pyautogui.press("enter")

                button_logout = await find_element_center(
                    "assets/emsys/button_logout.png", (0, 0, 130, 150), 60
                )

                if button_logout:
                    console.print(
                        "Login realizado com sucesso.", style="bold green"
                    )
                    return {
                        "sucesso": True,
                        "retorno": "Logou com sucesso no emsys!",
                    }
        else:
            log_msg = "Elemento de login não está pronto."
            logger.info(log_msg)
            console.print(log_msg, style="bold red")
            return {"sucesso": False, "retorno": "Falha ao logar no EMSys!"}


async def api_simplifica(
    urlSimplifica: str,
    status: str,
    observacao: str,
    uuidsimplifica: str,
    numero_nota,
    valor_nota,
):

    data = {
        "uuid_simplifica": uuidsimplifica,
        "status": status,
        "numero_nota": numero_nota,
        "observacao": observacao,
        "valor_nota": valor_nota,
    }

    try:
        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=False)
        ) as session:
            async with session.post(f"{urlSimplifica}", data=data) as response:
                data = await response.text()
                log_msg = f"\nSucesso ao enviar {data}\n para o simplifica"
                console.print(
                    log_msg,
                    style="bold green",
                )
                logger.info(log_msg)

    except Exception as e:
        err_msg = f"Erro ao comunicar com endpoint do Simplifica: {e}"
        console.print(f"\n{err_msg}\n", style="bold green")
        logger.info(err_msg)


def add_start_on_boot_to_registry():
    import winreg as reg

    try:
        # Caminho para a chave Run
        registry_path = r"Software\Microsoft\Windows\CurrentVersion\Run"

        # Nome da chave
        key_name = "worker-startup"

        # Caminho para o executável no diretório atual
        directory_value = os.path.join(os.getcwd(), "worker-startup.bat")

        # Acessar a chave de registro
        registry_key = reg.OpenKey(
            reg.HKEY_CURRENT_USER, registry_path, 0, reg.KEY_SET_VALUE
        )

        # Adicionar ou modificar o valor
        reg.SetValueEx(registry_key, key_name, 0, reg.REG_SZ, directory_value)

        # Fechar a chave de registro
        reg.CloseKey(registry_key)

        log_msg = f"Chave {key_name} adicionada ao registro com sucesso com o valor '{directory_value}'!"
        console.print(
            f"\n{log_msg}\n",
            style="bold green",
        )
        logger.info(log_msg)

    except Exception as e:
        err_msg = f"Erro ao adicionar ao registro: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)


def create_worker_bat():
    try:
        # Caminho do diretório atual
        current_dir = os.getcwd()
        nome_arquivo = "worker-startup.bat"

        # Conteúdo do arquivo
        # cd %USERPROFILE%
        bat_content = f"""@echo off
cd {current_dir}        
start /min "" "worker" "run"
"""

        # Caminho completo para o arquivo
        bat_file_path = os.path.join(current_dir, nome_arquivo)

        # Escrevendo o conteúdo no arquivo
        with open(bat_file_path, "w") as file:
            file.write(bat_content.strip())

        log_msg = f"Arquivo {nome_arquivo} criado com sucesso em {bat_file_path}!"
        console.print(
            f"\n{log_msg}\n",
            style="bold green",
        )
        logger.info(log_msg)

    except Exception as e:
        err_msg = f"Erro ao criar o arquivo {nome_arquivo}: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)


def take_screenshot() -> Path:   
    screenshot_path = Path.cwd() / "temp" / "screenshot.png"      
    screenshot_path.parent.mkdir(parents=True, exist_ok=True)    
    screenshot = pyautogui.screenshot()
    screenshot.save(screenshot_path)
    
    return screenshot_path


def preprocess_image(image_path):
    # Carregar a imagem
    image = cv2.imread(str(image_path))

    # Converter para escala de cinza
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    # Aplicar threshold binário
    _, binary_image = cv2.threshold(gray, 150, 255, cv2.THRESH_BINARY)

    # Remover ruído com medianBlur
    denoised_image = cv2.medianBlur(binary_image, 3)

    # Aumentar o contraste
    contrast_image = cv2.convertScaleAbs(denoised_image, alpha=1.5, beta=0)

    return contrast_image


def take_target_position(
    screenshot_path: Path, target_text: str, vertical=0, horizontal=0
) -> tuple | None:

    selected_image = Image.open(screenshot_path).convert("L")

    # Configurações do pytesseract
    # custom_config = r'--oem 3 --psm 6'

    # Extrair dados do texto usando pytesseract
    text_data = pytesseract.image_to_data(
        selected_image,
        output_type=pytesseract.Output.DICT,
        lang="por",  # , config=custom_config
    )

    # Identificar a posição do texto desejado
    field_center = None
    for i, text in enumerate(text_data["text"]):
        if len(text) > 0:
            if target_text.lower() in str(text).lower():
                x = text_data["left"][i]
                y = text_data["top"][i]
                w = text_data["width"][i]
                h = text_data["height"][i]
                # Centralizando nas coordenadas do campo
                field_center = (x + w // 2, y + h // 2)
                break

    # Aplicar as modificações de posição
    if field_center:
        field_center = (field_center[0] + horizontal, field_center[1] + vertical)

    return field_center


def find_target_position(
    screenshot_path: Path,
    target_text: str,
    vertical_pos: int = 0,
    horizontal_pos: int = 0,
    attempts: int = 5,
) -> tuple | None:
    print("Dentro da função: ", screenshot_path)
    attempt = 0
    while attempt < attempts:
        target_pos = take_target_position(
            screenshot_path,
            target_text,
            vertical=vertical_pos,
            horizontal=horizontal_pos,
        )
        console.print(f"Tentativa {attempt + 1} - Posição: {target_pos}")
        attempt += 1
        time.sleep(1)
        if target_pos is not None:
            log_msg = f"Posição do campo [{target_text}] encontrada na tentativa [{attempt}], com valor de: {target_pos}"
            console.print(log_msg, style="green")
            logger.info(log_msg)
            break

    if target_pos == None:
        log_msg = f"Não foi possível encontrar o campo [{target_text}] em [{attempts}] tentativas!"
        console.print(log_msg, style="red")
        logger.error(log_msg)

    return target_pos

def select_model():
    time.sleep(5)
    screenshot_path = take_screenshot()
    field = find_target_position(screenshot_path, "Documento", 0, 140, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Documento'"}
    pyautogui.click(field)
    time.sleep(1)
    pyautogui.write("Nfe")
    pyautogui.hotkey("enter")
    #Procura o tipo de documento "NFe - NOTA FISCAL ELETRONICA PROPRIA - DANFE SERIE 077"
    for _i in range(3):
        pyautogui.hotkey("down")
    time.sleep(2)
    return {"sucesso": True, "retorno": f"Modelo Selecionado"}

def import_nfe():
    screenshot_path = take_screenshot()
    field = find_target_position(screenshot_path, "Importar", 0, 0, 15) 
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo de 'Importar NFe'"}
    pyautogui.click(field)
    time.sleep(8)
    return {"sucesso": True, "retorno": f"Clicou Importar nfe"}

def select_nfe(nfe_type):
    screenshot_path = take_screenshot()
    field = find_target_position(screenshot_path, nfe_type, 0 ,0, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo de 'Notas de Outras Empresas'"}
    pyautogui.click(field)

    return {"sucesso": True, "retorno": f"Selecionou {nfe_type}"}



import time
import warnings

import pyautogui
from pywinauto.application import Application
from rich.console import Console

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    find_target_position,
    kill_process,
    login_emsys,
    select_model,
    select_nfe,
    take_screenshot,
    type_text_into_field,
)

pyautogui.PAUSE = 0.5
console = Console()



async def entrada_de_notas(task):
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        #Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)
       
        #Seta config entrada na var nota para melhor entendimento
        nota = task['configEntrada']

        #Abre um novo emsys
        await kill_process("EMSys")
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config['conConfiguracao'], app, task)

        if return_login['sucesso'] == True:
            type_text_into_field('Nota Fiscal de Entrada', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            time.sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login["retorno"]}")
            console.print(f"\nError Message: {return_login["retorno"]}", style="bold red")
            return return_login
        
        #Procura campo documento
        model  = select_model()
        if model['sucesso'] == True:
            console.log(model['message'], style='bolde green')
        else:
            return {"sucesso": False, "retorno": f"{model['message']}"}

        #Clica em 'Importar-Nfe'
        import_nfe  = import_nfe()
        if import_nfe['sucesso'] == True:
            console.log(import_nfe['message'], style='bolde green')
        else:
            return {"sucesso": False, "retorno": f"{import_nfe['message']}"}

        #Seleciona 'Notas de outras empresas'
        nfe_select = select_nfe("Empresas")
        if nfe_select['sucesso'] == True:
            console.log(nfe_select['message'], style='bolde green')
        else:
            return {"sucesso": False, "retorno": f"{nfe_select['message']}"}
        #Clica em  'OK' para selecionar
        pyautogui.click(970, 666)
        time.sleep(3)

        #Tela importa nota de outra empresa
        #Seleciona Empresa externa
        screenshot_path = take_screenshot()
        field = find_target_position(screenshot_path, "de", 30, 0, 15)
        if field == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Empresa'"}
        pyautogui.click(field)
        pyautogui.write(str(int(nota['cnpjFornecedor'][8:12])))
        pyautogui.hotkey('tab')

        #Digita datas de emissão
        field = find_target_position(screenshot_path, "emissão", 0, 40, 15)
        if field == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Data de emissão inicio'"}
        pyautogui.click(field)
        pyautogui.write(nota['dataEmissao'].replace('/', ''))

        field = find_target_position(screenshot_path, "a", 0, 40, 15)
        if field == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Data de emissão fim'"}
        pyautogui.click(field)
        pyautogui.write(nota['dataEmissao'].replace('/', ''))

        #Digita numero da nota
        field_num = find_target_position(screenshot_path, "Núm", 0, 60, 15)
        if field == None:
           return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Núm Nota'"}
        pyautogui.click(field)
        pyautogui.write(nota['numeroNota'])

        #Clica em pesquisar
        field = find_target_position(screenshot_path, "pesquisar", 0 ,0, 15)
        if field == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o botão 'Pesquisar'"}
        pyautogui.click(field)
        time.sleep(120)
  
        #clica na nota
        pyautogui.click(791,483)
        time.sleep(2)

        #Clica em 'Importar'
        field = find_target_position(screenshot_path, "cancelar",0 ,100 , 15)
        if field == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o botão importar"}
        pyautogui.click(field)
        
        #Tela "Infromações para importação da Nota Fiscal Eletrônica"
        screenshot_path = take_screenshot()

        if nota['existeDespesa'] == 'Sim':
            #Digita natureza da operação
            pyautogui.hotkey('tab')
            pyautogui.write("1152")
            pyautogui.hotkey('tab')

            #Digita tipo despesa
            field = find_target_position(screenshot_path, "Despesa", 15, 40, 15)
            if field == None:
                return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo despesa"}
            pyautogui.click(field)
            pyautogui.write("291")
            pyautogui.hotkey("tab")

            #Marca "Manter Natureza de operação selecionada"
            pyautogui.click(963, 578)
            #Marca "Manter Calculo PIS/COFINS"
            pyautogui.click(705, 667)

        #Clica em ok
        pyautogui.click(1100, 730)
        time.sleep(60)

        #TODO fazer tratativa de "itens com ncm divergente" - PASSO 27.1 DA IT
        #TODO Itens não localizados - PASSO 27.2 DA IT
        #TODO trativa "Selecionar itens fornecedor" - PASSO 27.3 da it

        screenshot_path = take_screenshot()
        #Seleciona pagamento
        field = find_target_position(screenshot_path, "pagamento", 0, 0, 15)
        if field == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo de seleção de pagamento"}
        pyautogui.click(field)
        time.sleep(1)
        screenshot_path = take_screenshot()
        #Seleciona "Espécie de Caixa"
        field = find_target_position(screenshot_path, "Espécie", 0, 100, 15)
        if field == None:
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo para digitar o valor"}
        pyautogui.click(field)
        pyautogui.write("27")
        pyautogui.hotkey("enter")
    
        #Digita "Valor"
        field = find_target_position(screenshot_path, "Documento", 0, 200, 15)
        pyautogui.doubleClick(field)
        pyautogui.hotkey("ctrl", "a")
        pyautogui.hotkey("del")
        pyautogui.write(nota['valorNota'])

        #Clica para lançar a nota
        pyautogui.click(594, 299)
        time.sleep(5)

        #Variação Maxima de custo
        pyautogui.click(1234, 633)

        #Quantidade de tempo alta devido o emsys ser lento
        time.sleep(120)

        screenshot_path = take_screenshot()
        message_nota = find_target_position(screenshot_path, "incluída", 0, 0, 15)
        if message_nota != None:
            pyautogui.click(960, 565)
            return {"sucesso": True, "retorno": f"Nota Lançada"}
        
        screenshot_path = take_screenshot()
        rateio_window = find_target_position(screenshot_path, "rateio", 0, 0, 15)
        if rateio_window != None:
            pyautogui.click(rateio_window)
            screenshot_path = take_screenshot()
            
            #Clica em Selecionar todos
            field = find_target_position(screenshot_path, "todos", 0, 0, 15)
            if field == None:
                return {"sucesso": False, "retorno": f"Campo 'Selecionar Todos' não encontrado"}
            pyautogui.click(field)
            time.sleep(2)
            
            #Digita "Centro" 1000 + filialEmpresaOrigem
            field = find_target_position(screenshot_path, "centro", 0, 85, 15)
            if field == None:
                return {"sucesso": False, "retorno": f"Campo centro não localizado"}
            pyautogui.click(field)
            filial = 1000 + int(nota['filialEmpresaOrigem'])
            pyautogui.write(str(filial))
            pyautogui.hotkey("tab")

            #Marca "Aplicar rateio aos itens selecionados"
            pyautogui.hotkey("space")
            pyautogui.hotkey("tab")

            #Digita % Rateio
            pyautogui.hotkey("ctrl", "a")
            pyautogui.hotkey("del")
            pyautogui.write("100")

            #Clica Incluir registro
            pyautogui.click(1161, 548)
            
            #Clica OK
            pyautogui.click(1200, 683)
            time.sleep(5)

        #Aviso CFOP
        screenshot_path = take_screenshot()
        field = None
        field = find_target_position(screenshot_path, "mesmo", 0, 0, 15)
        if field == None:
            console.print("Nota com CFOP correto")
        else:
            #Clica em OK
            pyautogui.click(1096, 603)
            #Clica em "Principal"
            field = find_target_position(screenshot_path, "Principal", 0, 0, 15)
            pyautogui.click(field)
            #Selecio no "Principal" a NOP correta
            field = find_target_position(screenshot_path, "Inscrição", 20, 0, 15)
            pyautogui.click(field)
            pyautogui.write("1152")
            #Clica para lançar a nota
            pyautogui.click(594, 299)

            #Clica novamente em ok na variação de custo      
            field = find_target_position(screenshot_path, "ultrapassam", 0, 0, 15)
            if field == None:
                console.print("Nota sem itens com variação de custo")
            else:
                pyautogui.click(1234, 633)
                
            
        screenshot_path = take_screenshot()
        message_nota = find_target_position(screenshot_path, "incluída", 0, 0, 15)
        if message_nota != None:
            pyautogui.click(960, 565)
            return {"sucesso": True, "retorno": f"Nota Lançada"}
        else:
            return {"sucesso": False, "retorno": f"Erro ao lançar nota"}

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}



import os
import subprocess
from pathlib import Path

import requests
import toml
from packaging import version
from rich.console import Console
from typer import Exit

from worker_automate_hub.utils.logger import logger

console = Console()


def update_version_in_toml(file_path, new_version):
    try:
        # Abrir e carregar o arquivo TOML
        with open(file_path, "r") as file:
            config = toml.load(file)

        # Alterar a versão
        config["params"]["version"] = new_version

        # Salvar as alterações de volta no arquivo TOML
        with open(file_path, "w") as file:
            toml.dump(config, file)

        console.print(f"Versão atualizada para {new_version} no arquivo {file_path}")
    except Exception as e:
        console.print(f"Erro ao atualizar a versão: {e}")

def create_update_script(new_version):
    try:
        current_dir = Path.cwd()

        # Caminho do arquivo TOML
        toml_file_path = os.path.join(current_dir, 'settings.toml')

        # Comando para fechar a aplicação
        close_command = 'taskkill /F /IM worker.exe'

        # Comando para atualizar o pacote via pipx
        update_command = 'pipx upgrade worker-automate-hub --force'

        # Comando para atualizar a versão no arquivo TOML
        update_toml_command = f'python -c "import toml; config=toml.load(\'{toml_file_path}\'); config[\'params\'][\'version\'] = \'{new_version}\'; toml.dump(config, open(\'{toml_file_path}\', \'w\'))"'

        # Comando para reiniciar a aplicação
        restart_command = f'start cmd /K "cd /D {current_dir} && worker run --force"'

        # Caminho do arquivo de log
        log_file_path = os.path.join(current_dir, 'update.log')

        # Criando o conteúdo do script .bat
        script_content = f"""
        @echo off
        {close_command}
        {update_command}
        if %errorlevel% equ 0 (
            echo Update successful, updating version in TOML file...
            {update_toml_command}
            echo Version updated successfully, restarting the application...
            {restart_command}
        ) else (
            echo Update failed. > "{log_file_path}"
            echo Error code: %errorlevel% >> "{log_file_path}"
            echo See pipx output for more details. >> "{log_file_path}"
        )
        """

        # Caminho completo para o arquivo update.bat
        bat_file_path = os.path.join(current_dir, 'update.bat')

        # Escrevendo o conteúdo no arquivo update.bat
        with open(bat_file_path, 'w') as file:
            file.write(script_content.strip())

        log_msg = "Arquivo de update criado com sucesso!"
        logger.info(log_msg)
        console.print(f"\n{log_msg}\n", style="bold green")
        
        return bat_file_path

    except Exception as e:
        err_msg = "Erro ao criar o arquivo de update: {e}"
        logger.error(err_msg)
        console.print(f"\n{err_msg}\n", style="bold red")


def run_update_script(bat_file_path):
    try:
        # Executando o script .bat
        subprocess.run([bat_file_path], check=True, shell=True)
        console.print("\nupdate.bat executado com sucesso!\n", style="bold green")
        logger.info("update.bat executado com sucesso!")
        raise Exit(code=0)
    except subprocess.CalledProcessError as e:
        console.print(f"\nErro ao executar o update.bat: {e}\n", style="bold red")
        logger.error(f"Erro ao executar o update.bat: {e}")

def get_installed_version(package_name):
    try:
        result = subprocess.run(['pipx', 'list'], capture_output=True, text=True, check=True)
        lines = result.stdout.splitlines()
        for line in lines:
            if package_name in line:
                # Parse the version from the line, assuming the line format is 'package_name <version>'
                return line.split(' ')[5].replace(',', '')
    except subprocess.CalledProcessError as e:
        err_msg = f"Ocorreu um erro ao obter a versão do pacote instalada: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)
    return None

def check_for_update():
    try:
        """Check if there is a new version of the package available on PyPI."""
        package_name = "worker-automate-hub"
        current_version = get_installed_version(package_name)

        response = requests.get(f"https://pypi.org/pypi/{package_name}/json")
        latest_version = response.json()
        console.print("last: "+latest_version["info"]["version"])
        console.print("current: "+current_version)
        if version.parse(latest_version["info"]["version"]) > version.parse(current_version):
            console.print(
                f"Uma nova versão [bold cyan]({latest_version["info"]["version"]})[/bold cyan] está disponível. Atualizando..."
            )
            bat_file_path = create_update_script(latest_version["info"]["version"])
            run_update_script(bat_file_path)
        else:            
            current_dir = Path.cwd()

            # Caminho do arquivo TOML
            toml_file_path = os.path.join(current_dir, 'settings.toml')
            update_version_in_toml(toml_file_path, current_version)
            console.print("\nVocê está usando a versão mais atualizada.\n", style="green")
    except Exception as e:
        console.print(f"\nErro ao verificar novas versões do pacote: {e}\n", style="bold red")
        logger.error(f"Erro ao verificar novas versões do pacote: {e}")



import asyncio
from time import sleep

from worker_automate_hub.utils.util import (
    find_target_position,
    take_screenshot,
)


async def playground():
    sleep(5)

    screen = take_screenshot()
    print(screen)
    res = find_target_position(screen, "su")
    print(res)


if __name__ == "__main__":
    asyncio.run(playground())



import aiohttp
import requests
from rich.console import Console

from worker_automate_hub.api.helpers.api_helpers import handle_api_response
from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import get_new_task_info, get_system_info

console = Console()


async def get_new_task():
    env_config, _ = load_env_config()
    try:
        

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}
        data = await get_new_task_info()

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.post(
                f"{env_config["API_BASE_URL"]}/robo/new-job",
                data=data,
                headers=headers_basic,
            ) as response:
                return await handle_api_response(response)

    except Exception as e:
        err_msg = f"Erro ao obter nova tarefa: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None

async def burnQueue(id_fila: str):
    env_config, _ = load_env_config()
    try:       

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}
        

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.delete(
                f"{env_config["API_BASE_URL"]}/fila/burn-queue/{id_fila}",
                headers=headers_basic,
            ) as response:
                if response.status == 200:
                    logger.info("Fila excluida com sucesso.")
                    console.print("\nFila excluida com sucesso.\n", style="bold green")    
                else:
                    logger.error(f"Erro ao excluir a fila: {response.content}") 
                    console.print(f"Erro ao excluir a fila: {response.content}", style="bold red")          

    except Exception as e:
        err_msg = f"Erro remover registro da fila: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None
    
async def notify_is_alive():
    env_config, _ = load_env_config()
    try:       

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}
        data = await get_system_info()

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.put(
                f"{env_config["API_BASE_URL"]}/robo/last-alive",
                data=data,
                headers=headers_basic,
            ) as response:
                return await handle_api_response(response, last_alive=True)

    except Exception as e:
        err_msg = f"Erro ao informar is alive: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None
    
async def get_processo(uuidProcesso: str):
    env_config, _ = load_env_config()
    try:      
        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}       
        

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/processo/{uuidProcesso}",                
                headers=headers_basic,
            ) as response:
                return await response.json()

    except Exception as e:
        err_msg = f"Erro ao obter o processo: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None


async def get_workers():
    env_config, _ = load_env_config()
    try:
        

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/robo/workers",
                headers=headers_basic,
            ) as response:
                return await response.json()

    except Exception as e:
        err_msg = f"Erro ao obter a lista de workers: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None

async def get_config_by_name(name: str):
    env_config, _ = load_env_config()
    try:        

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/configuracao/{name}",
                headers=headers_basic,
            ) as response:
                return await response.json()

    except Exception as e:
        err_msg = f"Erro ao obter a configuração: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None
    
async def send_gchat_message(message: str):
    env_config, _ = load_env_config()
    try:       

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}        

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.post(
                f"{env_config["API_BASE_URL"]}/google-chat",
                data={"message": message},
                headers=headers_basic,
            ) as response:
                data = await response.text()
                console.print(f"Retorno de enviar msg no chat: {data}")
                # return await response.json()

    except Exception as e:
        err_msg = f"Erro ao enviar mensagem ao Google Chat: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None
    
async def unlock_queue(id: str):
    env_config, _ = load_env_config()
    try:      
        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}       
        

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/fila/unlock-queue/{id}",                
                headers=headers_basic,
            ) as response:
                return await response.text()

    except Exception as e:
        err_msg = f"Erro ao desbloquear a fila: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None


def read_secret(path: str, vault_token: str):
    

    url = f"https://aspirina.simtech.solutions/{path}"
    headers = {"X-Vault-Token": vault_token, "Content-Type": "application/json"}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        result = response.json()
        return result["data"]["data"]
    elif response.status_code == 403:
        err_msg = "403 - Token inválido!"
        logger.error(err_msg)
        console.print(f"\n{err_msg}\n", style="bold red")
    else:
        response.raise_for_status()


def load_environments(env: str, vault_token: str):

    environments = {}   
    credentials = {}

    environments[env] = read_secret(path=f"v1/{env}-sim/data/worker-automate-hub/env", vault_token=vault_token)
    credentials[env] = read_secret(path=f"v1/{env}-sim/data/worker-automate-hub/credentials.json", vault_token=vault_token)

    return environments[env], credentials[env]



import asyncio
import importlib.metadata

import inquirer
import psutil
from inquirer.themes import GreenPassion
from rich.console import Console
from typer import Context, Exit, Option, Typer

from worker_automate_hub.api.client import get_workers, load_environments
from worker_automate_hub.core.so_manipulation import (
    add_worker_config,
    download_assets_and_extract_from_drive,
    download_tesseract,
    write_env_config,
)
from worker_automate_hub.utils.updater import check_for_update
from worker_automate_hub.utils.util import (
    add_start_on_boot_to_registry,
    create_worker_bat,
)

from .worker import run_worker

console = Console()
app = Typer()

HELP_MESSAGE = """
[b]Worker[/] - Grupo Argenta

Forma de uso: [b]worker [SUBCOMANDO] [ARGUMENTOS][/]

Existem 3 subcomandos disponíveis para essa aplicação

- [b]run[/]: Inicializa o Worker na máquina atual e começa a solicitar trabalho para o orquestrador.
- [b]validate[/]: Verifica se o Worker atual está configurado corretamente e pronto para ser inicializado.
- [b]assets[/]: Realiza a limpeza e depois download na pasta assets de todos arquivos utilizado pelo worker durante execução.

[b]Exemplos de uso:[/]
 [b][blue]RUN[/][/]
    [green][b]worker[/][/] [b]run[/]

 [b][blue]UPDATE[/][/]
    [green][b]worker[/][/] [b]update[/]

 [b][blue]VALIDATE[/][/]
    [green][b]worker[/][/] [b]validate[/]

---

[b]Help:[/]
 [b]Para mais informações[/]
    [green][b]worker[/][/] --help

 [b]Para ver a versão instalada[/]
    [green][b]worker[/][/] --version

 [b]Para gerar o arquivo de configuração[/]
    [green][b]worker[/][/] --configure

 [b]Para informações detalhadas
    [blue][link=https://github.com/SIM-Rede/worker-automate-hub]Repo no GIT Argenta[/][/] | [blue][link=https://pypi.org/project/worker-automate-hub/]Publicação no PyPI[/][/]
"""


def function_help(flag: bool):
    if flag:
        console.print(
            importlib.metadata.version("worker-automate-hub"),
            style="bold blue",
        )
        raise Exit(code=0)


def function_configure(flag: bool):

    if flag:
        console.clear()
        environment_names = [
            "local",
            "qa",
            "main",
        ]
        q = [
            inquirer.Text("vault_token", "Por favor digite o token do Vault"),
            inquirer.List("env_list", "Selecione o ambiente", environment_names),
        ]
        r = inquirer.prompt(q, theme=GreenPassion())

        env_sel, credentials = load_environments(r["env_list"], r["vault_token"])
        write_env_config(env_sel, credentials)
        workers = asyncio.run(get_workers())

        if workers == None:
            console.print("\nNenhum worker encontrado.\n", style="yellow")
            raise Exit(code=0)
        else:

            nomes_workers = [worker["nomRobo"] for worker in workers]
            q2 = [
                inquirer.List(
                    "worker_list", "Selecione um Worker", choices=nomes_workers
                )
            ]
            r2 = inquirer.prompt(q2, theme=GreenPassion())
            worker_sel = next(
                worker for worker in workers if worker["nomRobo"] == r2["worker_list"]
            )
            add_worker_config(worker_sel)

            q3 = [
                inquirer.Confirm(
                    "reg_config",
                    message="Adicionar configuração de inicialização aos registros do Windows?",
                )
            ]
            r3 = inquirer.prompt(q3, theme=GreenPassion())

            if r3["reg_config"]:
                add_start_on_boot_to_registry()

            q4 = [
                inquirer.Confirm(
                    "assets_config",
                    message="Atualizar a pasta assets?",
                )
            ]
            r4 = inquirer.prompt(q4, theme=GreenPassion())

            if r4["assets_config"]:
                asyncio.run(download_assets_and_extract_from_drive())

            q5 = [
                inquirer.Confirm(
                    "worker_bat", message="Criar o arquivo worker-startup.bat?"
                )
            ]
            r5 = inquirer.prompt(q5, theme=GreenPassion())

            if r5["worker_bat"]:
                create_worker_bat()

            q6 = [
                inquirer.Confirm(
                    "tesseract_install", message="Iniciar a instalação do Tesseract?"
                )
            ]
            r6 = inquirer.prompt(q6, theme=GreenPassion())

            if r6["tesseract_install"]:
                asyncio.run(download_tesseract())

            console.print(
                "\nConfiguração finalizada com sucesso!\n", style="bold green"
            )

            raise Exit(code=0)


@app.callback(invoke_without_command=True)
def main(
    ctx: Context,
    version: bool = Option(False, callback=function_help, is_flag=True),
    configure: bool = Option(False, callback=function_configure, is_flag=True),
):
    if ctx.invoked_subcommand:
        return
    console.print(HELP_MESSAGE)


def is_command_running(command):
    """
    Verifica se um comando CLI está sendo executado em outro terminal.

    :param command: O comando CLI a ser verificado (lista de strings).
    :return: True se o comando estiver sendo executado, False caso contrário.
    """
    command_str = " ".join(command)
    rep = 0
    for proc in psutil.process_iter(["pid", "name", "cmdline"]):
        try:
            cmdline = proc.info["cmdline"]
            if cmdline and isinstance(cmdline, list):
                cmdline_str = " ".join(cmdline)
                if command_str in cmdline_str:
                    print(cmdline_str)
                    rep += 1

        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
            continue

    if rep > 1:
        return True
    else:
        return False


from typer import Option


@app.command()
def run(
    force: bool = Option(
        False,
        "--force",
        help="Força a execução mesmo se outro comando já estiver rodando.",
    )
):
    command = ["worker", "run"]
    if not force and is_command_running(command):
        console.print(
            "\nO script já está em execução. Saindo...\n", style="bold yellow"
        )
        raise Exit(code=0)
    else:
        asyncio.run(run_worker())


@app.command()
def update():
    check_for_update()



from datetime import datetime

from pytz import timezone
from rich.console import Console

from worker_automate_hub.api.client import get_processo, unlock_queue
from worker_automate_hub.api.rpa_historico_service import store, update
from worker_automate_hub.models.dto.rpa_hitorico_dto import (
    RpaHistoricoDTO,
    RpaHistoricoStatusEnum,
)
from worker_automate_hub.tasks.task_definitions import task_definitions
from worker_automate_hub.utils.logger import logger

console = Console()


async def perform_task(task):
    console.print(f"\nProcesso a ser executado: {task['nomProcesso']}\n", style="green")
    logger.info(f"Processo a ser executado: {task['nomProcesso']}")
    task_uuid = task["uuidProcesso"]
    processo = await get_processo(task_uuid)
    historico = await _store_historico(task, processo)
    try:
        if task_uuid in task_definitions:
            result = await task_definitions[task_uuid](task)
            await _update_historico(
                historico["success"]["uuidHistorico"],
                task,
                result["sucesso"],
                result,
                processo,
            )
            return result
        else:
            console.print(f"Processo não encontrado: {task_uuid}", style="yellow")
            logger.error(f"Processo não encontrado: {task_uuid}")
            await _update_historico(
                historico["success"]["uuidHistorico"],
                task,
                False,
                {"sucesso": False, "retorno": f"Processo não encontrado: {task_uuid}"},
                processo,
            )
            await unlock_queue(task["uuidFila"])
            return None
    except Exception as e:
        console.print(f"Erro ao performar o processo: {e}\n", style="red")
        logger.error(f"Erro ao performar o processo: {e}")
        await _update_historico(
            historico["success"]["uuidHistorico"],
            task,
            False,
            {"sucesso": False, "retorno": f"Erro ao performar o processo: {e}"},
            processo,
        )


async def _store_historico(task: dict, processo: dict):
    try:
        from worker_automate_hub.config.settings import load_worker_config

        worker_config = load_worker_config()
        tz = timezone("America/Sao_Paulo")
        start_time = datetime.now(tz).isoformat()

        # Armazenar início da operação no histórico
        start_data = RpaHistoricoDTO(
            uuidProcesso=task["uuidProcesso"],
            uuidRobo=worker_config["UUID_ROBO"],
            prioridade=processo["prioridade"],
            desStatus=RpaHistoricoStatusEnum.Processando,
            configEntrada=task["configEntrada"],
            datInicioExecucao=start_time,
            datEntradaFila=task["datEntradaFila"],
        )

        store_response = await store(start_data)
        console.print(f"\nHistorico salvo: {store_response}")
        return store_response
    except Exception as e:
        console.print(f"Erro ao salvar o histórico do processo: {e}\n", style="red")
        logger.error(f"Erro ao salvar o histórico do processo: {e}")


async def _update_historico(
    historico_uuid: str,
    task: dict,
    sucesso: bool,
    retorno_processo: dict,
    processo: dict,
):
    try:
        from worker_automate_hub.config.settings import load_worker_config

        worker_config = load_worker_config()
        tz = timezone("America/Sao_Paulo")
        des_status = (
            RpaHistoricoStatusEnum.Sucesso if sucesso else RpaHistoricoStatusEnum.Falha
        )
        end_time = datetime.now(tz).isoformat()

        # Armazenar fim da operação no histórico
        end_data = RpaHistoricoDTO(
            uuidHistorico=historico_uuid,
            uuidProcesso=task["uuidProcesso"],
            uuidRobo=worker_config["UUID_ROBO"],
            prioridade=processo["prioridade"],
            desStatus=des_status,
            configEntrada=task["configEntrada"],
            retorno=retorno_processo,
            datFimExecucao=end_time,
        )

        update_response = await update(end_data)
        console.print(f"\nHistorico atualizado: {update_response}")
        return update_response

    except Exception as e:
        console.print(f"Erro ao atualizar o histórico do processo: {e}\n", style="red")
        logger.error(f"Erro ao atualizar o histórico do processo: {e}")



import asyncio

import pyfiglet
from rich.console import Console

from worker_automate_hub.api.client import (
    burnQueue,
    get_new_task,
    notify_is_alive,
    send_gchat_message,
)
from worker_automate_hub.config.settings import (
    load_env_config,
    load_worker_config,
)
from worker_automate_hub.tasks.task_definitions import is_uuid_in_tasks
from worker_automate_hub.tasks.task_executor import perform_task
from worker_automate_hub.utils.logger import logger

console = Console()


async def check_and_execute_tasks():
    while True:
        try:
            task = await get_new_task()
            worker_config = load_worker_config()
            processo_existe = await is_uuid_in_tasks(task["data"]['uuidProcesso'])
            if processo_existe:
                await burnQueue(task["data"]["uuidFila"])
                logger.info(f"Executando a task: {task['data']['nomProcesso']}")
                await perform_task(task["data"])
            else:
                log_message = f"O processo [{task["data"]['nomProcesso']}] não existe no Worker [{worker_config["NOME_ROBO"]}] e não foi removido da fila."
                logger.error(log_message)
                await send_gchat_message(log_message)
            await asyncio.sleep(5)
        except Exception as e:
            logger.error(f"Ocorreu um erro de execução: {e}")
            await asyncio.sleep(5)


async def notify_alive():
    env_config, _ = load_env_config()
    while True:
        try:
            logger.info("Notificando last alive...")
            await notify_is_alive()
            await asyncio.sleep(int(env_config["NOTIFY_ALIVE_INTERVAL"]))
        except Exception as e:
            logger.error(f"Erro ao notificar que está ativo: {e}")
            await asyncio.sleep(int(env_config["NOTIFY_ALIVE_INTERVAL"]))


async def main_process():
    env_config, _ = load_env_config()
    worker_config = load_worker_config()
   
    custom_font = "slant"
    ascii_banner = pyfiglet.figlet_format(f"Worker", font=custom_font)
    # console.clear()
    console.print(ascii_banner + f" versão: {env_config["VERSION"]}\n", style="bold blue")
    initial_msg = f"Worker em execução: {worker_config["NOME_ROBO"]}"
    logger.info(initial_msg)
    console.print(f"{initial_msg}\n", style="green")

    tasks = [check_and_execute_tasks(), notify_alive()]
    await asyncio.gather(*tasks)


def run_worker():
    try:
        while True:
            asyncio.run(main_process())
            break
    except asyncio.CancelledError:
        logger.info("Aplicação encerrada pelo usuário.")



import logging
import os
from pathlib import Path

from worker_automate_hub.config.settings import LOG_LEVEL


def setup_logger(name, log_file, level=logging.INFO):
    formatter = logging.Formatter("%(asctime)s %(levelname)s %(message)s")
    handler = logging.FileHandler(log_file)
    handler.setFormatter(formatter)
    logger = logging.getLogger(name)
    logger.setLevel(level)
    logger.addHandler(handler)
    return logger


current_dir = Path.cwd()
log_file_path = current_dir / "logs"
if not os.path.exists(log_file_path):
    os.makedirs(log_file_path)

logger = setup_logger("main_logger", f"{log_file_path}/app.log", int(LOG_LEVEL))



from worker_automate_hub.tasks.jobs.descartes import descartes
from worker_automate_hub.tasks.jobs.fazer_pudim import fazer_pudim
from worker_automate_hub.tasks.jobs.login_emsys import login_emsys
from worker_automate_hub.tasks.jobs.descartes import descartes
from worker_automate_hub.tasks.jobs.transferencias import transferencias
from worker_automate_hub.tasks.jobs.entrada_de_notas import entrada_de_notas


task_definitions = {
    "5b295021-8df7-40a1-a45e-fe7109ae3902": fazer_pudim,
    "a0788650-de48-454f-acbf-3537ead2d8ed": login_emsys,
    "abcfa1ba-d580-465a-aefb-c15ac4514407": descartes,
    "2c8ee738-7447-4517-aee7-ce2c9d25cea9": transferencias,
    "bf763394-918b-47be-bb36-7cddc81a8174": entrada_de_notas
}


async def is_uuid_in_tasks(uuid_to_check):
    """
    Verifica se um UUID está presente nas definições de tarefas.

    :param uuid_to_check: O UUID a ser verificado.
    :return: True se o UUID estiver presente, False caso contrário.
    """
    return uuid_to_check in task_definitions.keys()
